<?php
namespace Teknisa\MYO\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ControlePedidos extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'pedidoserv_pessoa_cliente';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $dataSourceManager;
    protected $messageProcedure;

    protected $lista_pedidos;

    const TABLE_LISTA  = "LISTA";
    const VALID_STATUS = array("APROVADO", "EMPRODUCAO", "FINALIZADO", "ENTREGUE");
    const NEXT_STATUS  = -1;


    public function __construct($crudService, ParameterBag $parameterBag, $entityManager, $newCode, $environment, $dataSourceManager, $messageProcedure) {
        parent::__construct($crudService, $parameterBag);

        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->messageProcedure = $messageProcedure;
    }

    /** 
     * generateOrCancelNF
     *
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     * 
     * @return Response $response
     */
    public function generateOrCancelNF (DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $orders = $rows[0];
        $operationType = $rows[1];

        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //Bloco de codigo feito para contornar incoerencia na procedure
            $procedureResultArray = array();

            if($operationType['OPERACAO'] === 'G'){
                $resultProcedure = $this->nfProcedure($orders, $operationType);
                $this->deleteLista($this->nrListaPedidos);

                array_push($procedureResultArray, $resultProcedure);
            } else {
                $resultProcedure = $this->nfProcedure($orders, $operationType);
                $this->deleteLista($this->nrListaPedidos);
                
                array_push($procedureResultArray, $resultProcedure);
            }

            $connection->commit();

            $response->addDataSet(new DataSet ('GERA_NF_MYORDER', $procedureResultArray));
        }catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    /**
     * insertLista
     * Executa o insert da lista
     *
     * @param Array $listaPedidos
     *
     * @return int|string
     */
    private function insertLista($listaPedidos) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $rows = array();
        foreach ($listaPedidos as $pedido) {
            $row = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $pedido['NRPEDIDOSERV'],
                "__is_new" => true
            );
            array_push($rows, $row);
        }
        $dataSet = new DataSet($dataSourceName, $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");

        $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rowsToSave);
        return $nrlista;
    }

    /**
     * deleteLista
     * Executa o delete da lista
     *
     * @param int|string $nrLista
     *
     * @return Void
     */
    private function deleteLista($nrlista) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $conditions = array(
            array(
                "columnName" => "NRLISTA",
                "operator"   => '=',
                "value"      => $nrlista
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $this->dataSourceManager->delete($dataSet);
    }

    /**
     * formatResultOfProcedure
     * Formata result da procedure adicionando informacoes relevantes
     * sobre o pedido
     *
     * @param Array $result
     * @param Array $orders
     *
     * @return Array
     */
    private function formatResultOfProcedure($result, $orders) {
        $result['ID'] = array();

        foreach ($orders as $order) {
            array_push($result['ID'], $order['NRPEDIDOSERV']);
        }

        return $result;
    }

    /**
     * nfProcedure
     * Executa a procedure no BD e retorna resultado
     *
     * @param Array $orders
     * @param Array $operationType
     *
     * @return Array
     */
    private function nfProcedure($orders, $operationType) {
        $this->nrListaPedidos = $this->insertLista($orders); 
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_NF_MYORDER');

        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAPEDIDO', Param::PARAM_INPUT, $this->nrListaPedidos, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPERACAO', Param::PARAM_INPUT, $operationType['OPERACAO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 400));
        $procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $operationType['OPERACAO'] === 'G' ? $orders[0]['CDTIPOOPER'] : NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTSAIDANF', Param::PARAM_INPUT, $operationType['OPERACAO'] === 'G' ? $orders[0]['DTFATURA'] : NULL, Param::PARAM_TYPE_STR));
        $result = $procedure->execute();
        
        return $this->formatResultOfProcedure($result, $orders);
    }

    /*
        Back order status
        Rules:
        	APROVADO   ->  null
            EMPRODUCAO -> APROVADO   : Valida existencia de Retirada para o pedido
            FINALIZADO -> EMPRODUCAO : Valida existencia de Nota Fiscal para o pedido
            ENTREGUE   -> FINALIZADO : Não permite mudanca
    */
    public function changeBackStatus(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $msgsError = array();
            $dataSave = array();
            
            foreach($dataset as $row){
                $nextKey = array_search($row["IDSTATUS"], self::VALID_STATUS) + self::NEXT_STATUS;
                if($nextKey >= 0 && $nextKey < count(self::VALID_STATUS) - 2){
                    $nextStatus = self::VALID_STATUS[$nextKey];
                    if($nextStatus == "APROVADO" && $this->existsRetirada($row)){
                        array_push($msgsError, array("Retirada", $row["NRPEDIDOSERV"]));
                    } else if($nextStatus == "EMPRODUCAO" && $this->existsNF($row)){
                        array_push($msgsError, array("Nota Fiscal", $row["NRPEDIDOSERV"]));
                    } else {
                        $row["IDSTATUS"] = $nextStatus;
                        $row["changed"]  = true;
                        array_push($dataSave, $row);
                    }
                }
            }
            if(!empty($dataSave)){
                $this->persistChangeBackStatus($dataSave);
            }
            $response->addDataSet(new DataSet ('error', $msgsError));
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function persistChangeBackStatus($dataSave){
        $this->dataSourceName = "pedidoserv";
        $dataSet = new DataSet($this->dataSourceName, $dataSave);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
        $this->crudService->save($this->dataSourceName, $rowsToSave);
    }

    private function translateMsgError($nrpedido, $case){
        return "Já exsite uma ".$case." gerada para o Pedido '".$nrpedido."'."; 
    }

    private function existsRetirada($row) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      =>  $this->environment->getNrorgTrab()
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $row["CDFILIAL"]
            ),
            array(
                "columnName" => "NRPEDIDOSERV",
                "operator"   => '=',
                "value"      => $row["NRPEDIDOSERV"]
            )
        );
        $rows = $this->findRows("itempedidoserv_cardadia_retirada", $conditions);
        return !empty($rows);
    }

    private function existsNF($row) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      =>  $this->environment->getNrorgTrab()
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $row["CDFILIAL"]
            ),
            array(
                "columnName" => "NRPEDIDOSERV",
                "operator"   => '=',
                "value"      => $row["NRPEDIDOSERV"]
            ),
            array(
                "columnName" => "NRLANCTONF",
                "operator"   => "IS_NOT_NULL",
                "value"      => null
            )
        );
        $rows = $this->findRows("pedidoserv", $conditions);
        return !empty($rows);
    }
    
    private function findRows($dataSourceName, $conditions){
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        return $dataSet->getRows();
    }

    public function generateRequisicaoExtra(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $procedureArrayError = array();
            
            $nrListaPedidos = $this->insertLista($rows); 
            $resultProcedure = $this->requisicaoExtraProcedure($connection, $nrListaPedidos);
            $this->deleteLista($nrListaPedidos);

            if(!empty($resultProcedure) && $resultProcedure['P_RETORNO'] != 0){
                $this->messageProcedure->concatErrorMessages($rows[0]['CDFILIAL'], $resultProcedure['P_RETORNO'], $response);
            }
            $connection->commit();
        }catch(\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function requisicaoExtraProcedure($connection, $pedidos) {
        $procedure = new StoredProcedure($connection, 'REQUISICAO_EXTRA_MYORDER');
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAPEDIDO', Param::PARAM_INPUT, $pedidos, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 400));
        $result = $procedure->execute();
        return $result;
    }
}